@@ -2,6 +2,64 @@ prm_cv <-
 function(X,y,a,fairct=4,opt="median",subset=NULL,
 	segments=10,segment.type="random",trim=0.2,sdfact=2,plot.opt=TRUE){
 
+# PRM Partial Robust M-regression estimator with CV
+# This version does not rely on mvr{chemometrics} which is always
+# mean-centering X; instead, "Unisimpls" is used, see below
+
+##################################################################
+Unisimpls <- function(X,y,a){
+  # univariate simpls; PF, 09.02.2010
+  # X ... explanatory variables
+  # y ... response variable
+  # a ... number of desired components
+  ##################################
+  n <-  nrow(X)
+  px <-  ncol(X)
+  # if n<px do SVD:
+  if (px > n) {
+    dimensions <- 1
+    dimension <- px - n
+    ressvd <- svd(t(X))
+    X <- ressvd$v %*% diag(ressvd$d)
+    n <- nrow(X)
+    px <- ncol(X)
+  }
+  else {
+    dimensions <- 0
+  }
+  # end of SVD
+  s <- t(X)%*%y
+  U <- matrix(0,nrow=n,ncol=a)
+  V <- matrix(0,nrow=px,ncol=a)
+  R <- matrix(0,nrow=px,ncol=a) # simpls weights
+  B <- V
+  for (j in 1:a) {  # extract one component after the other
+    r <- s
+    u <- X%*%r
+    u <- u-U[,1:max(1,j-1)]%*%(t(U[,1:max(1,j-1)])%*%u)
+    normu <- drop(sqrt(t(u)%*%u))
+    u <- u/normu
+    r <- r/normu
+    p <- t(X)%*%u
+    v <- p - V[,1:max(1,j-1)]%*%(t(V[,1:max(1,j-1)])%*%p)
+    v <- v/drop(sqrt(t(v)%*%v))
+    s <- s-v%*%(t(v)%*%s)
+    U[,j] <- u
+    R[,j] <- r
+    V[,j] <- v
+    B[,j] <- R[,1:j]%*%t(R[,1:j])%*%t(X)%*%y
+  }
+  # if SVD:
+  if (dimensions == 1) {
+    B <- ressvd$u %*% B
+  }
+  # end of SVD
+  list(coefficients=B,scores=U)
+} # end of Unisimpls
+##################################################################
+
+
+##################################################################
 # calls pcr_cvsub for cross validating prm (partial robust M-regression)
 prm_cvsub <- function(X,y,a,fairct,opt,subset,
 	segments,segment.type,trim){
@@ -26,14 +84,16 @@ ymc <- as.vector(scale(y, center = my, scale = FALSE))
 wx <- sqrt(apply(Xmc^2,1,sum))
 wx <- wx/median(wx)
 wx <- 1/((1+abs(wx/fairct))^2)
+
 wy <- abs(ymc)
 wy <- wy/median(wy)
 wy <- 1/((1+abs(wy/fairct))^2)
+
 w <- wx*wy
 Xw <- Xmc*sqrt(w)
 yw <- ymc*sqrt(w)
 
-require(pls)
+#require(pls)
 
 # start CV
 ypred <- rep(NA,length(y))
@@ -52,9 +112,12 @@ for (n.seg in 1:length(segment)) {
         difference <- 1
         while ((difference>0.01) && loops<30){
           ngammaold <- ngamma
-          spls <- mvr(ywsel~Xwsel,data=dsel,ncomp=a,method="simpls")
-	  b <- spls$coef[,,a]
-	  gamma <- t(t(ymc[obsuse])%*%spls$sco)
+          ###spls <- mvr(ywsel~Xwsel,data=dsel,ncomp=a,method="simpls")
+	  spls <- Unisimpls(Xwsel,ywsel,a) # new
+	  ###b <- spls$coef[,,a]
+	  b <- spls$coef[,a] # new
+	  ###gamma <- t(t(ymc[obsuse])%*%spls$sco) # wrong
+	  gamma <- t(t(ywsel) %*% spls$sco)
 	  T <- spls$sco/sqrt(wsel)
 	  r <- ymc[obsuse]-T%*%gamma
 	  rc <- r-median(r)
@@ -75,28 +138,45 @@ for (n.seg in 1:length(segment)) {
              {
              wsel <- replace(wsel, list=w0, values=10^(-6))              
              }
-	  Xwsel <- Xmc[obsuse,]*sqrt(wsel)
-	  ywsel <- ymc[obsuse]*sqrt(wsel)
+	  ###Xwsel <- Xmc[obsuse,]*sqrt(wsel)
+	  Xwsel <- X[obsuse,]*sqrt(wsel)
+	  ###ywsel <- ymc[obsuse]*sqrt(wsel)
+	  ywsel <- y[obsuse]*sqrt(wsel)
 	  loops <- loops+1
         }
 
+########################################################################
+# NEW PF:
 # predictions in each segment
-ypred[-obsuse] <- drop(as.matrix(Xmc[-obsuse,]))%*%b
+b0 <- drop(t(mx)%*%b)
+ypred[-obsuse] <- drop(as.matrix(Xmc[-obsuse,]))%*%b+b0
 # residuals in each segment
-yresj <- ymc[-obsuse]-ypred[-obsuse]
+yresj <- y[-obsuse]-ypred[-obsuse]
 SEPj[n.seg] <- sd(yresj)
 absresj=abs(yresj)
 resjlarge=quantile(absresj,1-trim)
 SEPtrimj[n.seg] <- sd(yresj[absresj<=resjlarge])
 }
-res <- ymc-ypred
-medres <- median(res)
-res <- res-medres
-ypred <- ypred+medres
-# ======= NEU BL: ===========
-# y-Medianzentrierung "rückgängig", sonst stimmt ypred nicht
-        ypred <- ypred + my
-# ==========================
+res <- y-ypred
+########################################################################
+## OLD: predictions in each segment
+#ypred[-obsuse] <- drop(as.matrix(Xmc[-obsuse,]))%*%b
+## residuals in each segment
+#yresj <- ymc[-obsuse]-ypred[-obsuse]
+#SEPj[n.seg] <- sd(yresj)
+#absresj=abs(yresj)
+#resjlarge=quantile(absresj,1-trim)
+#SEPtrimj[n.seg] <- sd(yresj[absresj<=resjlarge])
+#}
+#res <- ymc-ypred
+#medres <- median(res)
+#res <- res-medres
+#ypred <- ypred+medres
+## ======= NEU BL: ===========
+## y-Medianzentrierung "rückgängig", sonst stimmt ypred nicht
+#        ypred <- ypred + my
+## ==========================
+########################################################################
 SEPall=sd(res)
 absres=abs(res)
 reslarge=quantile(absres,1-trim)
@@ -104,7 +184,9 @@ SEPtrim=sd(res[absres<=reslarge])
 
 list(predicted=ypred,residuals=res,SEPall=SEPall,SEPtrim=SEPtrim,
 	SEPj=SEPj,SEPtrimj=SEPtrimj)
-}
+} # end of prm_cvsub
+#####################################################################
+
 
 #####################################################################
 error.bars <-
@@ -117,10 +199,21 @@ function (x, upper, lower, width = 0.02, ...)
     segments(x - barw, lower, x + barw, lower, ...)
     range(upper, lower)
 }
-
 #####################################################################
 
 
+#####################################################################
+# start of real routine:
+n=nrow(X)
+p=ncol(X)
+
+if (p>n){
+        ressvd <- svd(t(X))
+        X <- ressvd$v%*%diag(ressvd$d)
+        n=nrow(X)
+        p=ncol(X)
+}
+
 SEPall=rep(NA,a)
 SEPtrim=rep(NA,a)
 pred=matrix(NA,nrow=length(y),ncol=a)
